'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));

function render(props, obj) {
  var fn = props.children || props.render;
  if (fn == null) return null;
  return typeof fn === 'function' ? fn(obj) : fn;
}

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();







var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};



var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};











var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var AnyValue = function (_React$Component) {
  inherits(AnyValue, _React$Component);

  function AnyValue(props) {
    classCallCheck(this, AnyValue);

    var _this = possibleConstructorReturn(this, (AnyValue.__proto__ || Object.getPrototypeOf(AnyValue)).call(this, props));

    _this.transform = function (fn) {
      _this.setState(function (existing) {
        var next = fn(existing.value);
        return { value: next };
      }, function () {
        var onChange = _this.props.onChange;

        if (onChange) onChange(_this.state.value);
      });
    };

    _this.set = function (next) {
      _this.transform(function (v) {
        return typeof next === 'function' ? next(v) : next;
      });
    };

    _this.reset = function () {
      _this.transform(function () {
        return _this.state.initial;
      });
    };

    _this.clear = function () {
      _this.set();
    };

    var initial = void 0;
    if (props.defaultValue !== undefined) initial = props.defaultValue;
    if (props.value !== undefined) initial = props.value;
    var controlled = props.value !== undefined;
    _this.state = { controlled: controlled, initial: initial, value: initial };
    return _this;
  }

  createClass(AnyValue, [{
    key: 'render',
    value: function render$$1() {
      var set$$1 = this.set,
          reset = this.reset,
          clear = this.clear,
          props = this.props,
          value = this.value;

      return render(props, { value: value, set: set$$1, reset: reset, clear: clear });
    }
  }, {
    key: 'value',
    get: function get$$1() {
      var state = this.state,
          props = this.props;

      return state.controlled ? props.value : state.value;
    }
  }]);
  return AnyValue;
}(React.Component);

function defaults$1(props, fallback) {
  var value = props.value,
      defaultValue = props.defaultValue;

  if (value === undefined && defaultValue === undefined) defaultValue = fallback;
  return { value: value, defaultValue: defaultValue };
}

function proxy(value, method, mutates) {
  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    value.set(function (v) {
      var ret = v[method].apply(v, args);
      return mutates ? v : ret;
    });
  };
}

var ArrayValue = function ArrayValue(props) {
  return React.createElement(
    AnyValue,
    _extends({}, props, defaults$1(props, [])),
    function (value) {
      return render(props, _extends({}, value, {
        first: value.value[0],
        last: value.value[Math.max(0, value.value.length - 1)],
        clear: function clear() {
          return value.set([]);
        },
        concat: proxy(value, 'concat'),
        fill: proxy(value, 'fill'),
        filter: proxy(value, 'filter'),
        flat: proxy(value, 'flat'),
        flatMap: proxy(value, 'flatMap'),
        map: proxy(value, 'map'),
        reverse: proxy(value, 'reverse'),
        sort: proxy(value, 'sort'),
        slice: proxy(value, 'slice'),
        push: proxy(value, 'push', true),
        pop: proxy(value, 'pop', true),
        shift: proxy(value, 'shift', true),
        unshift: proxy(value, 'unshift', true),
        splice: proxy(value, 'splice', true)
      }));
    }
  );
};

var BooleanValue = function BooleanValue(props) {
  return React.createElement(
    AnyValue,
    _extends({}, props, defaults$1(props, false)),
    function (value) {
      return render(props, _extends({}, value, {
        clear: function clear() {
          return value.set(false);
        },
        toggle: function toggle() {
          return value.set(function (v) {
            return !v;
          });
        }
      }));
    }
  );
};

var DateValue = function DateValue(props) {
  return React.createElement(
    AnyValue,
    _extends({}, props, defaults$1(props, new Date())),
    function (value) {
      var d = value.value;
      var date = d.getDate();
      var hours = d.getHours();
      var milliseconds = d.getMilliseconds();
      var minutes = d.getMinutes();
      var month = d.getMonth();
      var seconds = d.getSeconds();
      var year = d.getFullYear();

      var setDate = proxy(value, 'setDate', true);
      var setHours = proxy(value, 'setHours', true);
      var setMilliseconds = proxy(value, 'setMilliseconds', true);
      var setMinutes = proxy(value, 'setMinutes', true);
      var setMonth = function setMonth(m) {
        return value.set(function (v) {
          return setMonthHelper(v, m);
        });
      };
      var setSeconds = proxy(value, 'setSeconds', true);
      var setYear = proxy(value, 'setFullYear', true);

      var incrementDate = function incrementDate() {
        var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
        return value.set(function (v) {
          v.setDate(date + n);
          return v;
        });
      };
      var incrementHours = function incrementHours() {
        var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
        return incrementMinutes(n * 60);
      };
      var incrementMilliseconds = function incrementMilliseconds() {
        var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
        return value.set(function (v) {
          return new Date(v.getTime() + n);
        });
      };
      var incrementMinutes = function incrementMinutes() {
        var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
        return incrementSeconds(n * 60);
      };
      var incrementMonth = function incrementMonth() {
        var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
        return value.set(function (v) {
          return addMonthsHelper(v, n);
        });
      };
      var incrementSeconds = function incrementSeconds() {
        var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
        return incrementMilliseconds(n * 1000);
      };
      var incrementYear = function incrementYear() {
        var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
        return incrementMonth(n * 12);
      };

      return render(props, _extends({}, value, {
        clear: function clear() {
          return value.set(new Date());
        },

        date: date,
        hours: hours,
        milliseconds: milliseconds,
        minutes: minutes,
        month: month,
        seconds: seconds,
        year: year,

        setDate: setDate,
        setHours: setHours,
        setMilliseconds: setMilliseconds,
        setMinutes: setMinutes,
        setMonth: setMonth,
        setSeconds: setSeconds,
        setYear: setYear,

        incrementDate: incrementDate,
        incrementHours: incrementHours,
        incrementMilliseconds: incrementMilliseconds,
        incrementMinutes: incrementMinutes,
        incrementMonth: incrementMonth,
        incrementSeconds: incrementSeconds,
        incrementYear: incrementYear,

        decrementDate: function decrementDate() {
          var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
          return incrementDate(0 - n);
        },
        decrementHours: function decrementHours() {
          var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
          return incrementHours(0 - n);
        },
        decrementMilliseconds: function decrementMilliseconds() {
          var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
          return incrementMilliseconds(0 - n);
        },
        decrementMinutes: function decrementMinutes() {
          var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
          return incrementMinutes(0 - n);
        },
        decrementMonth: function decrementMonth() {
          var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
          return incrementMonth(0 - n);
        },
        decrementSeconds: function decrementSeconds() {
          var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
          return incrementSeconds(0 - n);
        },
        decrementYear: function decrementYear() {
          var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
          return incrementYear(0 - n);
        },

        // Provide the years as `*FullYear` as well, since the native JavaScript
        // APIs are named like that for backwards compatibility.
        setFullYear: setYear,
        incrementFullYear: incrementYear,
        decrementFullYear: function decrementFullYear() {
          var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
          return incrementYear(0 - n);
        }
      }));
    }
  );
};

function setMonthHelper(date, month) {
  var year = date.getFullYear();
  var day = date.getDate();
  var desired = new Date(0);
  desired.setFullYear(year, month, 15);
  desired.setHours(0, 0, 0, 0);
  var max = getDaysInMonth(desired);
  date.setMonth(month, Math.min(day, max));
  return date;
}

function addMonthsHelper(date, amount) {
  var year = date.getFullYear();
  var desiredMonth = date.getMonth() + amount;
  var desired = new Date(0);
  desired.setFullYear(year, desiredMonth, 1);
  desired.setHours(0, 0, 0, 0);
  var max = getDaysInMonth(desired);
  date.setMonth(desiredMonth, Math.min(max, date.getDate()));
  return date;
}

function getDaysInMonth(date) {
  var year = date.getFullYear();
  var monthIndex = date.getMonth();
  var lastDayOfMonth = new Date(0);
  lastDayOfMonth.setFullYear(year, monthIndex + 1, 0);
  lastDayOfMonth.setHours(0, 0, 0, 0);
  return lastDayOfMonth.getDate();
}

var MapValue = function MapValue(props) {
  return React.createElement(
    AnyValue,
    _extends({}, props, defaults$1(props, new Map())),
    function (value) {
      return render(props, _extends({}, value, {
        set: function set$$1() {
          for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          return args.length === 1 ? value.set.apply(value, args) : value.set(function (v) {
            return v.set.apply(v, args);
          });
        },
        unset: proxy(value, 'delete', true),
        delete: proxy(value, 'delete', true),
        clear: proxy(value, 'clear', true)
      }));
    }
  );
};

var NumberValue = function NumberValue(props) {
  return React.createElement(
    AnyValue,
    _extends({}, props, defaults$1(props, 0)),
    function (value) {
      return render(props, _extends({}, value, {
        clear: function clear() {
          return value.set(0);
        },
        increment: function increment() {
          var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
          return value.set(function (v) {
            return v + n;
          });
        },
        decrement: function decrement() {
          var n = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
          return value.set(function (v) {
            return v - n;
          });
        }
      }));
    }
  );
};

var SetValue = function SetValue(props) {
  return React.createElement(
    AnyValue,
    _extends({}, props, defaults$1(props, new Set())),
    function (value) {
      var add = proxy(value, 'add');
      var remove = proxy(value, 'delete', true);
      var clear = proxy(value, 'clear', true);
      return render(props, _extends({}, value, {
        add: add,
        remove: remove,
        delete: remove,
        clear: clear,
        toggle: function toggle(val, boolean) {
          return boolean ? add(val) : remove(val);
        }
      }));
    }
  );
};

var ArrayValue$2 = function ArrayValue(props) {
  return React.createElement(
    AnyValue,
    _extends({}, props, defaults$1(props, [])),
    function (value) {
      return render(props, _extends({}, value, {
        clear: function clear() {
          return value.set('');
        },
        concat: proxy(value, 'concat'),
        normalize: proxy(value, 'normalize'),
        padEnd: proxy(value, 'padEnd'),
        padStart: proxy(value, 'padStart'),
        repeat: proxy(value, 'repeat'),
        replace: proxy(value, 'replace'),
        slice: proxy(value, 'slice'),
        substr: proxy(value, 'substr'),
        substring: proxy(value, 'substring'),
        toLowerCase: proxy(value, 'toLowerCase'),
        toUpperCase: proxy(value, 'toUpperCase'),
        trim: proxy(value, 'trim'),
        trimEnd: proxy(value, 'trimEnd'),
        trimStart: proxy(value, 'trimStart')
      }));
    }
  );
};

exports.AnyValue = AnyValue;
exports.ArrayValue = ArrayValue;
exports.BooleanValue = BooleanValue;
exports.DateValue = DateValue;
exports.MapValue = MapValue;
exports.NumberValue = NumberValue;
exports.SetValue = SetValue;
exports.StringValue = ArrayValue$2;
//# sourceMappingURL=react-values.js.map
